/*
 * 	BMP180_I2C.c
 *
 *  Created on: 21.12.2014
 *  Author: Gerrit Buhe, DL9GFA
 */

#include <msp430.h>
#include <stdint.h>
#include "BMP180_I2C.h"
#include "UCBxI2C.h"

extern uint8_t RxBuffer[4];
extern uint8_t TxBuffer[4];
extern volatile uint8_t RPT_Flag;			//enables repeated start when 1
extern uint8_t *pTxData;            		//pointer to TX data
extern uint8_t *pRxData;            	    //pointer to RX data
extern int8_t RxByteCtr,TxByteCtr;

struct BMP180_caldata						//read out of sensors EEPROM while initialization
{
	int16_t  AC1;
	int16_t  AC2;
	int16_t  AC3;
	uint16_t AC4;
	uint16_t AC5;
	uint16_t AC6;
	int16_t  B1;
	int16_t  B2;
	int16_t  MB;
	int16_t  MC;
	int16_t  MD;							//last sensor specific calibration data
	int16_t	 B5;							//no real caldata but needed from temp calculation for pressure calculation
} caldata;


//*********************************************************************************************************************
uint8_t BMP180_presentDetect( void)
{
	uint8_t timeout;

	while( UCB0STAT & UCBBUSY);				//wait if any I2C communication is still in progress
	UCB0I2CSA = BMP180_ADDRESS;				//set slave address to read from
	TxByteCtr = 1;							//send register address to read from
	pTxData   = TxBuffer;    				//start of auxilary TX buffer
	*pTxData  = 0xD0;	    				//put ID register address to auxilary TX buffer
	RPT_Flag  = 1;							//send without stop condition
	UCB0CTL1 |= UCTR + UCTXSTT;             //I2C TX, start condition
	timeout   = 10;
	while( UCB0CTL1 & UCTXSTT);				//wait until start condition and address is sent
	while( UCB0STAT & UCNACKIFG)			//is slave address acknowledged?
	{
		timeout--;
		if( timeout == 0)
		{
			UCB0CTL1 |= UCTXSTP;          	//stop condition to release bus
			return( 0);
		}
	}

	RxByteCtr = 1;
	pRxData   = RxBuffer;  					//start of RX buffer
	UCB0CTL1 &= ~UCTR;						//RX mode now
	UCB0CTL1 |= UCTXSTT;                    //I2C start condition
	while( UCB0STAT & UCBBUSY);				//wait until ID register is read
	if( RxBuffer[0] == 0x55)				//ID register should be 0x55 !
		return( 1);
	else
		return( 0);
}


//*********************************************************************************************************************
void BMP180_init( void)
{
	uint16_t *calPtr;
	uint16_t aux, i;

	UCBxI2C_read( BMP180_ADDRESS, 0xAA, 22, (uint8_t *) &caldata);
	while( UCB0STAT & UCBBUSY);				//wait until all cal data are read

	calPtr = (uint16_t *) &caldata;
	for( i=11; i>0; i-- )					//byte swap of all data needed! :o(
	{
		aux = *calPtr;						//read cal data word
		aux = (aux >> 8) | (aux << 8);		//swap bytes
		*calPtr++ = aux;					//store back, increment ptr to next data word
	}
}


//*********************************************************************************************************************
void BMP180_reset( void)
{
	while( UCB0STAT & UCBBUSY);				//wait if any I2C communication is still in progress
	TxBuffer[2] = 0xB6;										//command for reset
	UCBxI2C_write( BMP180_ADDRESS, 0xE0, 1, &TxBuffer[2]);	//send command
}


//*********************************************************************************************************************
void BMP180_startTempMeasurement( void)
{
	while( UCB0STAT & UCBBUSY);				//wait if any I2C communication is still in progress
	TxBuffer[2] = 0x2e;										//command for temp measurement
	UCBxI2C_write( BMP180_ADDRESS, 0xF4, 1, &TxBuffer[2]);	//send command, wait for 4.5ms before read result!
}


//*********************************************************************************************************************
int16_t BMP180_getTempMeasurement( void)
{
	long UT, X1, X2;

	UCBxI2C_read( BMP180_ADDRESS, 0xF6, 2, RxBuffer);		//reads result of temp measurement
	while( UCB0STAT & UCBBUSY);								//wait until results available in buffer

	UT = ((long)RxBuffer[0] << 8) + (long)RxBuffer[1];		//merge bytes
	X1 = ((UT - caldata.AC6) * caldata.AC5) >> 15;
	X2 = ((long)caldata.MC << 11) / (X1 + caldata.MD);
	caldata.B5 = (uint16_t)(X1 + X2);
	return( (int16_t)((caldata.B5 + 8) >> 4) );				//return temperature in 0.1°C
}


//*********************************************************************************************************************
void BMP180_startPressureMeasurement( void)
{
	while( UCB0STAT & UCBBUSY);				//wait if any I2C communication is still in progress
	TxBuffer[2] = 0x34 + (BMP180_OSS<<6);	//command for pressure measurement
	UCBxI2C_write( BMP180_ADDRESS, 0xF4, 1, &TxBuffer[2]);	//send command, wait for 25.5ms before read result!
}


//*********************************************************************************************************************
uint16_t BMP180_getPressureMeasurement( void)
{
	int32_t  UP, X1, X2, X3, B3, B6, p;
	uint32_t B4, B7;

	UCBxI2C_read( BMP180_ADDRESS, 0xF6, 3, RxBuffer);		//reads result of temp measurement
	while( UCB0STAT & UCBBUSY);								//wait until results available in buffer

	UP = (((int32_t)RxBuffer[0] << 16) + ((int32_t)RxBuffer[1] << 8) + (int32_t)RxBuffer[2]) >> (8-BMP180_OSS);	//merge bytes, correct by oversampling
															//UP is uncompensated pressure, make absolute value in Pascal out of it now
	B6 = (int32_t)caldata.B5 - 4000;
	X1 = ((int32_t)caldata.B2 * ((B6 * B6)>>12)) >> 11;
	X2 = ((int32_t)caldata.AC2 * B6) >> 11;
	X3 = X1 + X2;
	B3 = ((((int32_t)caldata.AC1*4 + X3) << BMP180_OSS) + 2) >> 2;
	X1 = ((int32_t)caldata.AC3 * B6) >> 13;
	X2 = ((int32_t)caldata.B1 * ((B6 * B6) >> 12)) >> 16;
	X3 = ((X1 + X2) + 2) >> 2;
	B4 = ((int32_t)caldata.AC4 * (uint32_t)(X3 + 32768)) >> 15;
	B7 = ((uint32_t)UP - B3) * (50000 >> BMP180_OSS);
	if( B7 < 0x80000000)
		p = (B7 << 1) / B4;
	else
		p = (B7 / B4 ) << 1;
	X1 = (p >> 8) * (p >> 8);
	X1 = (X1 * 3038) >> 16;
	X2 = (-7357 * p) >> 16;
	p  = p + ((X1 + X2 + 3791) >> 4);
	p = (p<<11) + (p<<10) + (p<<7) + (p<<6) + (p<<3) + (p<<2) + p;	//manual to avoid floating point operation for p = p * 0.1
	p = (p>>15);

	return( (uint16_t) (p));								//return pressure in 0.1 hPa
}

